import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class IRKeypad 
  extends JPanel 
  implements ActionListener, MouseListener {
  
  private static class ButtonInfo {
    int code;
    String name;
    int x;
    int y;
    ButtonInfo(String n, int c, int x_, int y_)
      { code = c; name = n; x = x_; y = y_; }
  }

  /** set of ButtonInfo */
  private Set buttons_;  

  /**  String (command) -> Integer (IR code) */
  private Map commands_; 

  /** set of String names of buttons that are down */
  private Set downButtons_;  

  private javax.swing.Timer repeatTimer_; 

  /** Hz */
  private static final int repeatRate_ = 11;

  public IRKeypad() {
    super();
    
    listeners_ = new Vector();
    buttons_ = new HashSet();
    commands_ = new HashMap();

    downButtons_ = new HashSet();
    repeatTimer_ = new javax.swing.Timer(1000 / repeatRate_, this);

    addButtonInfo(buttons_);

    // assumes there is a button for every place in the grid
    setLayout(new GridLayout(11, 3));
    
    for (int y = 0; y < 11; y++) {
      for (int x = 0; x < 3; x++) {
	ButtonInfo bi = getButtonForXY(x, y);
	if (bi != null) {
	  JButton b = new JButton(bi.name);
	  b.addMouseListener(this);
	  add(b);
	  commands_.put(bi.name, new Integer(bi.code));
	}
	else {
	  add(new JPanel());
	}
      }
    }
  }

  public static interface IRListener {
    public void keyPressed(int code);
  }

  private Vector listeners_;
  public void addListener(IRListener l) {
    listeners_.add(l);
  }
    
  private void notifyListeners(int code) {
    for (int i = 0; i < listeners_.size(); i++)
      ((IRListener) listeners_.get(i)).keyPressed(code);
  }

  private void handleRepeatTimer() {
    for (Iterator i = downButtons_.iterator(); i.hasNext(); /* */) {
      String s = (String) i.next();
      doButtonDown(s);
    }
  }

  public void actionPerformed(ActionEvent e) { 
    if (e.getSource() == repeatTimer_)
      handleRepeatTimer();
  }

  private String getButtonCommand(MouseEvent e) {
    Object o = e.getSource();
    JButton b = null;
    try { b = (JButton) o; }
    catch (ClassCastException x) { return null; }
    String s = b.getText();
    if (!commands_.containsKey(s))
      return null;
    else
      return s;
  }

  public void mousePressed(MouseEvent e) {
    String s = getButtonCommand(e);
    if (s == null)
      return;
    if (!downButtons_.contains(s)) {
      if (downButtons_.size() == 0) {
	Util.check(!repeatTimer_.isRunning());
	repeatTimer_.start();
      }
      else
	Util.check(repeatTimer_.isRunning());
      downButtons_.add(s);
      doButtonDown(s);
    }    
  }

  public void mouseReleased(MouseEvent e) {
    String s = getButtonCommand(e);
    if (s == null)
      return;
    if (downButtons_.contains(s)) {
      downButtons_.remove(s);
      if (downButtons_.size() == 0) {
	Util.check(repeatTimer_.isRunning());
	repeatTimer_.stop();
      }
      else
	Util.check(repeatTimer_.isRunning());
    }
  }

  public void mouseClicked(MouseEvent e) {  }
  public void mouseEntered(MouseEvent e) {  }
  public void mouseExited(MouseEvent e) {  }

  private void doButtonDown(String n) {
    Integer i = (Integer) commands_.get(n);
    Util.check(i != null);
    notifyListeners(i.intValue());      
  }

  private ButtonInfo getButtonForXY(int x, int y) {
    for (Iterator i = buttons_.iterator(); i.hasNext(); ) {
      ButtonInfo bi = (ButtonInfo) i.next();
      if (bi.x == x && bi.y == y)
	return bi;
    }
    return null;
  }

  private void addButtonInfo(Set s) {
    s.add(new ButtonInfo("1"            , 0x7689f00f, 0, 5));
    s.add(new ButtonInfo("2"	        , 0x768908f7, 1, 5));
    s.add(new ButtonInfo("3"	        , 0x76898877, 2, 5));
    s.add(new ButtonInfo("4"	        , 0x768948b7, 0, 6));
    s.add(new ButtonInfo("5"	        , 0x7689c837, 1, 6));
    s.add(new ButtonInfo("6"	        , 0x768928d7, 2, 6));
    s.add(new ButtonInfo("7"	        , 0x7689a857, 0, 7));
    s.add(new ButtonInfo("8"	        , 0x76896897, 1, 7));
    s.add(new ButtonInfo("9"	        , 0x7689e817, 2, 7));
    s.add(new ButtonInfo("0"	        , 0x76899867, 1, 8));
    s.add(new ButtonInfo("Now Playing"  , 0x76897887, 0, 10));
    s.add(new ButtonInfo("Shuffle"      , 0x7689d827, 0, 9));
    s.add(new ButtonInfo("Rew"	        , 0x7689c03f, 0, 1));
    s.add(new ButtonInfo("Play"	        , 0x768910ef, 2, 2));
    s.add(new ButtonInfo("Fwd"	        , 0x7689a05f, 2, 1));
    s.add(new ButtonInfo("Add"	        , 0x7689609f, 0, 2));
    s.add(new ButtonInfo("Pause"	, 0x768920df, 1, 1));
    s.add(new ButtonInfo("Size"	        , 0x7689f807, 1, 10));
    s.add(new ButtonInfo("Sleep"	, 0x7689b847, 2, 9));
    s.add(new ButtonInfo("Up"	        , 0x7689e01f, 1, 2));
    s.add(new ButtonInfo("Left"	        , 0x7689906f, 0, 3));
    s.add(new ButtonInfo("Right"	, 0x7689d02f, 2, 3));
    s.add(new ButtonInfo("Down"	        , 0x7689b04f, 1, 4));
    s.add(new ButtonInfo("Brightness"	, 0x768904fb, 2, 10));
    s.add(new ButtonInfo("Repeat"	, 0x768938c7, 1, 9));
    s.add(new ButtonInfo("Power"	, 0x768940bf, 2, 0));
    s.add(new ButtonInfo("Vol-"	        , 0x768900ff, 0, 0));
    s.add(new ButtonInfo("Vol+"	        , 0x7689807f, 1, 0));
  }
}
