import java.awt.event.*;

public class StreamController 
  implements ActionListener {

  private static final int playerBufSz_ = 2 * 64 * 1024;
  private byte playerBuf_[];

  private int rptr_;
  private int wptr_;

  private boolean playing_;
  private javax.swing.Timer playTimer_;
  private int timerPeriod_ = 100; // msecs

  StreamController() {
    playerBuf_ = new byte[playerBufSz_];
    rptr_ = wptr_ = 0;
    playing_ = false;
    playTimer_ = new javax.swing.Timer(timerPeriod_, this);
  }

  public int getWPtr() { return wptr_; }
  public int getRPtr() { return rptr_; }

  public boolean isPlaying() { return playing_; }

  public void reset() {
    pause();
    rptr_ = 0;
  }

  public void pause() {
    if (!playing_)
      return;
    playing_ = false;
    playTimer_.stop();
  }

  public void resume() {
    if (playing_)
      return;
    playTimer_.start();
  }

  // all writes must be an even number of bytes (e.g. whole 16-bit words)
  public void write(int wptr, byte b[], int offset, int n) {
    // Util.dbg("XXX writing " + n + " bytes at " + Integer.toHexString(wptr));
    Util.check(offset >= 0);
    Util.check(n >= 0);
    Util.check(offset + n <= b.length);
    Util.check((n & 1) == 0);
    
    if (n > playerBufSz_) 
      Util.dbg("StreamController.write: got more bytes than can fit into buffer, extra bytes will overwrite initial bytes");
    
    for (int i = 0; i < n; i++) {
      playerBuf_[wptr % playerBufSz_] = b[i + offset];
      wptr++;
    }

  }

  public void showBufferUsage() {

  }

  public void actionPerformed(ActionEvent e) {
    // consume some bytes...
    // typical mp3: 3114213 bytes in 4 mins ~13 bytes/msec 
    int n = 13 * timerPeriod_;

    // check for underflow, i.e. trying to read more data than there is
    if ((rptr_ == wptr_) ||
	(rptr_ < wptr_ && (rptr_ + n) >= wptr_) ||
	(rptr_ > wptr_ && ((rptr_ + n) % playerBufSz_) >= wptr_)) {
      // Util.dbg("StreamController: data under-run while playing");
    }
    
    rptr_ = (rptr_ + n) % playerBufSz_;
  }

}
