import java.io.IOException;

public class VFDUnit 
  implements Display.ChangeListener {

  private Display        disp_;
  private VFDScreen      screen_;
  private VFDCharFactory charFact_;

  public static final int SIZE_NORMAL = VFDScreen.SIZE_NORMAL;
  public static final int SIZE_SMALL  = VFDScreen.SIZE_SMALL;
  public static final int SIZE_TINY   = VFDScreen.SIZE_TINY;

  public VFDUnit(int size) {
    disp_ = new Display();
    disp_.addChangeListener(this);

    screen_ = new VFDScreen(disp_.width(), disp_.height(), size);

    charFact_ = new VFDCharFactory();
    try { charFact_.loadFontFromFile("chars.txt"); }
    catch (IOException x) { Util.fail("VFDUnit.VFDUnit(): Couldn't read font file"); }

    updateScreen();
  }

  private void updateScreen() {
    screen_.setOn(disp_.isOn());
    screen_.setBrightness(disp_.brightness());

    for (int y = 0; y < disp_.height(); y++) {
      for (int x = 0; x < disp_.width(); x++) {
	byte b = disp_.getByteAt(x, y);
	VFDChar c = charFact_.getCharForCode(b & 0xff);
	if (c == null) 
	  c = charFact_.getCharForCode(0x94); // default...
	screen_.setChar(x, y, c == null ? new VFDChar() : c);
      }
    }
  }

  public void handleCodes(byte codes[], int offset, int n) {
    disp_.handleCodes(codes, offset, n);
  }

  public void displayChanged() {
    updateScreen();
    screen_.repaint();
  }

  public void addNewChar(int code, VFDChar c) {
    charFact_.addCharForCode(code, c);
  }

  public VFDScreen getScreen() { return screen_; }
}
